/*convex_hull;In mathematics, the convex hull or convex envelope for a set of points X in a real vector space V is the minimal convex set containing X.

The convex hull also has a linear-algebraic characterization: the convex hull of X is the set of all convex combinations of points in X.

In computational geometry, a basic problem is finding the convex hull for a given finite set of points in the plane.*/



#include<fstream>
#include<iostream>
#include"ezwin.h"						//EzWindows for graphics..
#include<cmath>							//in function cosine used the predifined function sqrt from this library
#include<assert.h>						//to use assertions
#include<cstdlib>		
#include"rect.h"						//to draw rectangle shape
#include"bitmap.h"						//for bitmap
#include"randint.h"						//for timer events
using namespace std;
SimpleWindow w1("convex hull",30.0,15.0,Position(5.0,3.0));	//global declarations of windows
SimpleWindow w2("convex hull",30.0,15.0,Position(5.0,3.0));


BitMap convexBmp(w1);						//declaration of bitmap

float x[100],y[100];
int n;

int cordie();							//prototypes of the required functions..
int display();
int convex_hull();
float cosine(Position pos);
float cross(Position a,Position b,Position c);

int ApiMain()
	{w1.Open();        					 //to open window
	assert(w1.GetStatus()==WindowOpen);
	
	Position center1= w1.GetCenter();			//for setting position of text in window
	Position u_l=center1 + Position(-1.0,-7.0);
	Position l_r=center1 + Position(1.0,-6.0);

	Position u_l1=center1 + Position(-1.0,-6.0);
	Position l_r1=center1 + Position(1.0,-5.0);

	Position u_l2=center1 + Position(-1.0,-5.0);
	Position l_r2=center1 + Position(1.0,-4.0);

	Position u_l3=center1 + Position(-1.0,-4.0);
	Position l_r3=center1 + Position(1.0,-3.0);

	Position u_l4=center1 + Position(-1.0,6.0);
	Position l_r4=center1 + Position(1.0,7.0);

	Position u_l5=center1 + Position(-4.0,-3.0);
	
	w1.RenderText(u_l,l_r,"CONVEX HULL",Black);						//for displaying messages in window..
	w1.RenderText(u_l1,l_r1,"For planar objects, i.e., lying in the plane,",Black);
	w1.RenderText(u_l2,l_r2,"the convex hull may be easily visualized by imagining an elastic band stretched open to encompass the given points (represented as pins);",Black);
	w1.RenderText(u_l3,l_r3,"  when released, it will assume the shape of the required convex hull.",Black);
	w1.RenderText(u_l4,l_r4,"enter 'c'in terminal to proceed..",Black);

	
	convexBmp.Load("convexhull_1.xpm");			
	assert(convexBmp.GetStatus()==BitMapOkay);
	
	convexBmp.SetPosition(u_l5);					//to set position of image
	convexBmp.Draw();
	char s;
	cin>>s;
	if(s=='c')
		{cordie();
		}
	display();
	return 0;
	}
//************************************************************
int ApiEnd()
    {
    w2.Close();
    w2.StopTimer();
    return 0;
    }
//***********************************************************

//***********************************************************

int cordie()								//FUNCTION FOR TAKING INPUT OF COORDINATES
	{w1.Close();
	int z;
	cout<<"ENTER 1 IF YOU WANT TO ENTER COORDINATES IN THE TERMINAL\n";
	cout<<"ENTER 2 IF YOU HAVE ENTERED COORDINATES IN THE FILE (format for entering is given in how to compile file in folder\n";
	cin>>z;
	if(z==1)
	{
	cout<<"how many points do you want to plot..??\n";
	cin>>n;
	int count=0,i;
	do
		{if(count==3)
			{cout<<"maximum number of invalid attempts exceeded.\nPROGRAM TERMINATES..\n";
			exit (0);
			}
		if(n<3)
			{cout<<"you must enter atleast three points to plot the convex hull\ntry again...\n";
			cin>>n;
			count++;
			}
		if(n>=3)
			{
			for(i=0;i<n;i++)
				{cout<<"enter the cordinates of the required points\ntype x cordinate first then y cordinate separated with a space\nthen press enter\n";
				cin>>x[i]>>y[i];
				}
			break;
			}
 
		}while(count<4);
	}
	else if(z==2)
		{
		cout<<"how many points did you enter in file?\n";
		cin>>n;
		int i;
		ifstream fin;
		fin.open("coordinates.dat");
		for(i=0;i<n;i++)
			{
			fin>>x[i]>>y[i];
			}
		fin.close();
			}
		else
			{
			cout<<"ERROR! TRY AGAIN\n";
			exit(0);
			}
		return 0;
	}
//***********************************************************
int display()								//FUNCTION TO DISPLAY POINTS IN WINDOW FOR 4 SECONDS
	{w2.Open();
	assert(w2.GetStatus()==WindowOpen);
	
	int j;
	for(j=0;j<n;j++)
		{RectangleShape points(w2,x[j],y[j],Black,0.1,0.1);	//to display the given points
		points.Draw();
		}
 	w2.SetTimerCallback(convex_hull);
	w2.StartTimer(4000);						//to start TIMER of 4 secs..it will call covex_hull function in 4 secs..

	return 0;
	}
//************************************************************
int convex_hull()							//USING GRAHAM SCAN ALGORITHM TO FIND CONVEX HULL(tried our best to create   our own logic based on graham scan..)
	{
	int i,k,min=100.0,count;
	for(i=0;i<n;i++)
		{if(y[i]<min)						//to isolate the point with lowest y cordinate
			{min=y[i];
			k=i;
			}
		else if(y[i]==min && i!=0)				// greater x cordinate must be selected
			{if(x[i]>x[k])
				{k=i;
				}
			}
		}
	Position poly[100];						//points of the convex hull
	Position sequence[100];						//arranged array
	Position point[100],temp;
	int q;
	for(q=0;q<k;q++)
		{point[q]=Position(x[q],y[q]);
		}
	for(q=k;q<n;q++)
		{point[q]=Position(x[q+1],y[q+1]);
		}
	float max_cos;
	int w,e,r;
	for(w=0;w<n-1;w++)						//sorting in decreasing order of cosines
		{max_cos=0;
		for(e=w;e<n-1;e++)
			{if(cosine(point[e])>max_cos)
				{max_cos=cosine(point[e]);
				r=e;
				}
			}
		temp=point[w];
		point[w]=point[r];
		point[r]=temp;
		}
	int g;
	for(g=0;g+1<n;g++)
		{sequence[g+1]=point[g];
		}
	sequence[0]= Position(x[k],y[k]);
	int t,u;							//to find cross product of required vectors
	for(t=0,u=1;t+2<n;t++)
		{if(cross(sequence[t],sequence[t+1],sequence[t+2])>=0)
			{poly[u]=sequence[t+1];
			u++;
			}
		}
	
	poly[0]=sequence[0];
	poly[u]=sequence[n-1];
	
	int n_points;
	n_points=u+1;
	
	w2.RenderPolygon(poly,n_points,Blue);				//creates a polygon representing the convex hull
	return 0;
	}
//********************************************************************
float cosine(Position pos)						//to find cosine of angle formed with the position vector
	{float cos,o,p;
	o=pos.GetXDistance();
	p=pos.GetYDistance();
	cos=o/sqrt(o*o + p*p);
	return cos;
	}
//******************************************************************
float cross(Position a,Position b,Position c)				//to compute the cross product
	{float c_product,x_a,x_b,x_c,y_a,y_b,y_c;
	x_a=a.GetXDistance();
	x_b=b.GetXDistance();
	x_c=c.GetXDistance();
	y_a=a.GetYDistance();
	y_b=b.GetYDistance();
	y_c=c.GetYDistance();
	
	c_product=(x_b-x_a)*(y_c-y_a)-(x_c-x_a)*(y_b-y_a);
	return c_product;
	}






















